/// <reference types="node" />
import { ScopedLocalStorage } from "../lib/ScopedLocalStorage";
import { AddressString, IntNumber, RegExpString } from "../types";
import { Web3Request } from "./Web3Request";
import { ArbitraryResponse, ChildRequestEthereumAccountsResponse, EthereumAddressFromSignedMessageResponse, RequestEthereumAccountsResponse, ScanQRCodeResponse, SignEthereumMessageResponse, SignEthereumTransactionResponse, SubmitEthereumTransactionResponse, Web3Response } from "./Web3Response";
import { WalletLinkUI, WalletLinkUIOptions } from "../provider/WalletLinkUI";
import { WalletLinkRelayEventManager } from "./WalletLinkRelayEventManager";
import { WalletLinkRelayAbstract } from "./WalletLinkRelayAbstract";
import { EthereumTransactionParams } from "./EthereumTransactionParams";
export interface WalletLinkRelayOptions {
    walletLinkUrl: string;
    version: string;
    darkMode: boolean;
    storage: ScopedLocalStorage;
    relayEventManager: WalletLinkRelayEventManager;
    walletLinkUIConstructor: (options: Readonly<WalletLinkUIOptions>) => WalletLinkUI;
}
export declare class WalletLinkRelay implements WalletLinkRelayAbstract {
    private static accountRequestCallbackIds;
    private readonly walletLinkUrl;
    private readonly storage;
    private readonly session;
    private readonly relayEventManager;
    private readonly connection;
    private ui;
    private appName;
    private appLogoUrl;
    constructor(options: Readonly<WalletLinkRelayOptions>);
    attachUI(): void;
    resetAndReload(): void;
    setAppInfo(appName: string, appLogoUrl: string | null): void;
    getStorageItem(key: string): string | null;
    setStorageItem(key: string, value: string): void;
    childRequestEthereumAccounts(childSessionId: string, childSessionSecret: string, dappName: string, dappLogoURL: string, dappURL: string): Promise<ChildRequestEthereumAccountsResponse>;
    requestEthereumAccounts(): Promise<RequestEthereumAccountsResponse>;
    signEthereumMessage(message: Buffer, address: AddressString, addPrefix: boolean, typedDataJson?: string | null): Promise<SignEthereumMessageResponse>;
    ethereumAddressFromSignedMessage(message: Buffer, signature: Buffer, addPrefix: boolean): Promise<EthereumAddressFromSignedMessageResponse>;
    signEthereumTransaction(params: EthereumTransactionParams): Promise<SignEthereumTransactionResponse>;
    signAndSubmitEthereumTransaction(params: EthereumTransactionParams): Promise<SubmitEthereumTransactionResponse>;
    submitEthereumTransaction(signedTransaction: Buffer, chainId: IntNumber): Promise<SubmitEthereumTransactionResponse>;
    scanQRCode(regExp: RegExpString): Promise<ScanQRCodeResponse>;
    arbitraryRequest(data: string): Promise<ArbitraryResponse>;
    sendRequest<T extends Web3Request, U extends Web3Response>(request: T): Promise<U>;
    private publishWeb3RequestEvent;
    private publishWeb3RequestCanceledEvent;
    private publishEvent;
    private handleIncomingEvent;
    private handleWeb3ResponseMessage;
    private invokeCallback;
}
